@@ -43,14 +43,16 @@
 
 
 static void
-yelp_info_populate_tree_for_subdir (const char *basedir, GNode *parent)
+yelp_info_read_info_dir (const char *basedir, GSList **info_list)
 {
 	DIR           *dirh;
 	struct dirent *dent;
+	YelpSection   *section;
 
 	dirh = opendir (basedir);
-	if (!dirh)
+	if (!dirh) {
 		return;
+	}
 
 	readdir (dirh);		/* skip . & .. */
 	readdir (dirh);
@@ -59,17 +61,21 @@ yelp_info_populate_tree_for_subdir (const char *basedir, GNode *parent)
 		char *ctmp = NULL;
 		char uribuf[128], titlebuf[128];
 
-		if (dent->d_name[0] == '.')
+		if (dent->d_name[0] == '.') {
 			continue;
+		}
 
 		do {
-			if (ctmp)
+			if (ctmp) {
 				*ctmp = '\0';
+			}
+			
 			ctmp = strrchr (dent->d_name, '.');
 		} while (ctmp && strcmp (ctmp, ".info"));
 
-		if (!ctmp)
+		if (!ctmp) {
 			continue;
+		}
 
 		*ctmp = '\0';
 
@@ -78,36 +84,68 @@ yelp_info_populate_tree_for_subdir (const char *basedir, GNode *parent)
 
 		g_snprintf (uribuf, sizeof (uribuf), "info:%s", dent->d_name);
 
-		g_node_append_data (parent, 
-				    yelp_section_new (YELP_SECTION_DOCUMENT,
-						      titlebuf, uribuf, 
-						      NULL, NULL));
+		section = yelp_section_new (YELP_SECTION_DOCUMENT,
+					    titlebuf, uribuf, 
+					    NULL, NULL);
+		
+		*info_list = g_slist_prepend (*info_list, section);
+						      
 	}
 
 	closedir (dirh);
 }
 
 gboolean
-yelp_info_init (GNode *tree)
+yelp_info_init (GNode *tree, GList **index)
 {
 	GNode       *root;
 	struct stat  stat_dir1;
 	struct stat  stat_dir2;
+	GSList      *info_list = NULL;
+	GSList      *node;
+	YelpSection *section;
+	YelpSection *index_section;
+	gchar       *index_name;
+	gchar       *index_uri;
 	
-	root = g_node_append_data (tree, 
+	stat ("/usr/info", &stat_dir1);
+	stat ("/usr/share/info", &stat_dir2);
+	
+	yelp_info_read_info_dir ("/usr/info", &info_list);
+	
+	if (stat_dir1.st_ino != stat_dir2.st_ino) {
+		yelp_info_read_info_dir  ("/usr/share/info", &info_list);
+	}
+
+	if (g_slist_length (info_list) <= 0) {
+		return FALSE;
+	}
+
+	root = g_node_append_data (tree,
 				   yelp_section_new (YELP_SECTION_CATEGORY,
 						     _("info"), NULL, 
 						     NULL, NULL));
 
-	stat ("/usr/info", &stat_dir1);
-	stat ("/usr/share/info", &stat_dir2);
-	
-	yelp_info_populate_tree_for_subdir ("/usr/info", root);
-	
-	if (stat_dir1.st_ino != stat_dir2.st_ino) {
-		yelp_info_populate_tree_for_subdir ("/usr/share/info", 
-						    root);
+	info_list = g_slist_sort (info_list, yelp_section_compare);
+
+	for (node = info_list; node; node = node->next) {
+		g_node_append_data (root, node->data);
+		section = YELP_SECTION (node->data);
+		
+		index_name = g_ascii_strdown (section->name, -1);
+		index_uri = g_strconcat ("index:", section->uri, NULL);
+
+		index_section = yelp_section_new (YELP_SECTION_INDEX,
+						  index_name, index_uri,
+						  section->reference, 
+						  section->scheme);
+		g_free (index_name);
+		g_free (index_uri);
+
+		*index = g_list_prepend (*index, index_section);
 	}
 
+	g_slist_free (info_list);
+
 	return TRUE;
 }
